@@ -1,4 +1,4 @@
-__version__ = '0.5.4'
+__version__ = '0.5.8'
 
 from yarl import URL
 from distutils.version import StrictVersion
@@ -7,15 +7,32 @@ import asyncio
 import aiohttp
 import re
 
+from multiaddr import Multiaddr
+from multiaddr.exceptions import ParseError
+from multiaddr.exceptions import StringParseError
+
 from aioipfs import api
 from aioipfs.exceptions import *  # noqa
+from aioipfs.apis import dag as dag_api
 from aioipfs.apis import pin as pin_api
+from aioipfs.apis import multibase as multibase_api
+from aioipfs.apis import pubsub as pubsub_api
+from aioipfs.apis import swarm as swarm_api
+
+
+RPC_API_DEFAULT_PORT = 5001
+
+
+class IpfsDaemonVersion(StrictVersion):
+    pass
 
 
 class AsyncIPFS(object):
     """
     Asynchronous IPFS API client
 
+    :param str maddr: The multiaddr for the IPFS daemon's RPC API
+        (this is the preferred method of specifying the node's address)
     :param str host: Hostname/IP of the IPFS daemon to connect to
     :param int port: The API port of the IPFS deamon
     :param int conns_max: Maximum HTTP connections for this client
@@ -23,25 +40,33 @@ class AsyncIPFS(object):
     :param int conns_max_per_host: Maximum per-host HTTP connections
         (default: 0)
     :param int read_timeout: Socket read timeout
+    :param str api_version': IPFS protocol version ('v0' by default)
+    :param bool debug: Enable client debugging traces
+    :param loop: force a specific asyncio event loop
     """
 
-    def __init__(self, host='localhost', port=5001, loop=None,
-                 conns_max=0, conns_max_per_host=0,
-                 read_timeout=0, api_version='v0',
-                 debug=False):
+    def __init__(self,
+                 host: str = 'localhost',
+                 port: int = RPC_API_DEFAULT_PORT,
+                 maddr=None,
+                 loop=None,
+                 conns_max: int = 0,
+                 conns_max_per_host: int = 0,
+                 read_timeout: int = 0,
+                 api_version: str = 'v0',
+                 debug: bool = False):
 
         self._conns_max = conns_max
         self._conns_max_per_host = conns_max_per_host
         self._read_timeout = read_timeout
-        self._host = host
-        self._port = port
         self._debug = debug
-
+        self._host, self._port = None, None
         self.loop = loop if loop else asyncio.get_event_loop()
 
-        self.api_url = URL.build(host=host, port=port,
-                                 scheme='http',
-                                 path='/api/{v}/'.format(v=api_version))
+        self._api_url = self.__compute_base_api_url(host=host,
+                                                    port=port,
+                                                    maddr=maddr,
+                                                    api_version=api_version)
 
         self.session = self.get_session()
 
@@ -49,12 +74,12 @@ class AsyncIPFS(object):
         self.core = api.CoreAPI(self)
         self.p2p = api.P2PAPI(self)
         self.block = api.BlockAPI(self)
-        self.pubsub = api.PubSubAPI(self)
+        self.pubsub = pubsub_api.PubSubAPI(self)
         self.bitswap = api.BitswapAPI(self)
         self.bootstrap = api.BootstrapAPI(self)
         self.cid = api.CidAPI(self)
         self.config = api.ConfigAPI(self)
-        self.dag = api.DagAPI(self)
+        self.dag = dag_api.DagAPI(self)
         self.dht = api.DhtAPI(self)
         self.diag = api.DiagAPI(self)
         self.file = api.FileAPI(self)
@@ -63,11 +88,13 @@ class AsyncIPFS(object):
         self.key = api.KeyAPI(self)
         self.pin = pin_api.PinAPI(self)
         self.log = api.LogAPI(self)
+        self.multibase = multibase_api.MultibaseAPI(self)
         self.name = api.NameAPI(self)
         self.object = api.ObjectAPI(self)
         self.refs = api.RefsAPI(self)
         self.repo = api.RepoAPI(self)
-        self.swarm = api.SwarmAPI(self)
+        self.routing = api.RoutingAPI(self)
+        self.swarm = swarm_api.SwarmAPI(self)
         self.tar = api.TarAPI(self)
         self.stats = api.StatsAPI(self)
 
@@ -81,6 +108,10 @@ class AsyncIPFS(object):
     def agent_version(self):
         return self._agent_version
 
+    @property
+    def api_url(self):
+        return self._api_url
+
     @property
     def host(self):
         return self._host
@@ -145,6 +176,54 @@ class AsyncIPFS(object):
     def commands(self):
         return self.core.commands
 
+    def __compute_base_api_url(self, host=None, port=None,
+                               maddr=None, api_version='v0'):
+        """
+        Compute and return the base kubo API url from the settings
+        passed to the constructor. If a multiaddr is passed, use that
+        to build the API URL, otherwise use host/port.
+
+        :rtype: URL
+        """
+
+        if isinstance(maddr, str) or isinstance(maddr, Multiaddr):
+            try:
+                # The Multiaddr constructor can handle a string or
+                # a Multiaddr instance
+                node_maddr = Multiaddr(maddr)
+
+                # Extract the host/port from the multiaddr protocols
+                for proto in node_maddr.protocols():
+                    if proto.name in ['ip4', 'ip6', 'dns4', 'dns6']:
+                        self._host = node_maddr.value_for_protocol(
+                            proto.name
+                        )
+                    elif proto.name == 'tcp':
+                        self._port = int(
+                            node_maddr.value_for_protocol(proto.name)
+                        )
+
+                assert isinstance(self._host, str)
+                assert isinstance(self._port, int)
+            except (StringParseError,
+                    ParseError,
+                    AssertionError) as perr:
+                raise InvalidNodeAddressError(
+                    f'Invalid kubo node multiaddr: {maddr}: {perr}'
+                )
+        elif isinstance(host, str) and isinstance(port, int):
+            self._host = host
+            self._port = port
+        else:
+            raise InvalidNodeAddressError()
+
+        return URL.build(
+            host=self._host,
+            port=self._port,
+            scheme='http',
+            path=f'/api/{api_version}/'
+        )
+
     def api_endpoint(self, path):
         # Returns a joined URL between the base API url and path
         return self.api_url.join(URL(path))
@@ -164,29 +243,35 @@ class AsyncIPFS(object):
                 comps = agent_version.rstrip('/').split('/')
 
                 if len(comps) >= 2:
-                    self._agent_version = comps[1]
+                    self._agent_version = IpfsDaemonVersion(
+                        re.sub(
+                            '(-.*$)', '', comps[1]
+                        )
+                    )
 
         return self.agent_version
 
-    async def agent_version_superioreq(self, version):
+    async def agent_version_superioreq(self, version: str):
         """
         Returns True if the node's agent version is superior or equal to
         version
 
-        :param str version: a go-ipfs version number e.g 0.4.10
+        :param str version: a kubo (formerly called go-ipfs)
+            version number e.g "0.14.0"
         """
         try:
             node_vs = await self.agent_version_get()
-            node_vs = re.sub('(-.*$)', '', node_vs)
-            version_node = StrictVersion(node_vs)
-            version_ref = StrictVersion(version)
-            return version_node >= version_ref
+            version_ref = IpfsDaemonVersion(version)
+            return node_vs >= version_ref
         except BaseException:
             return False
 
     async def agent_version_post0418(self):
         return await self.agent_version_superioreq('0.4.18')
 
+    async def agent_version_post011(self):
+        return await self.agent_version_superioreq('0.11.0')
+
     def get_session(self, conn_timeout=60.0 * 30, read_timeout=60.0 * 10):
         return aiohttp.ClientSession(
             connector=aiohttp.TCPConnector(
